@@ -12,6 +12,11 @@ fn main() {
     cfg = cfg.define("SNMALLOC_RUST_SUPPORT", "ON")
         .profile(build_type);
 
+    if cfg!(all(windows, target_env = "msvc")) {
+        cfg = cfg.define("CMAKE_CXX_FLAGS_RELEASE", "/MD /O2 /Ob2 /DNDEBUG");
+        cfg = cfg.define("CMAKE_C_FLAGS_RELEASE", "/MD /O2 /Ob2 /DNDEBUG");
+    }
+
     let target = if cfg!(feature = "1mib") {
         "snmallocshim-1mib-rust"
     } else {
@@ -27,12 +32,12 @@ fn main() {
     dst.push("./build");
 
     println!("cargo:rustc-link-lib={}", target);
-    if cfg!(unix) {
-        println!("cargo:rustc-link-search=native={}", dst.display());
-        println!("cargo:rustc-link-lib=dylib=stdc++");
-    } else {
+    if cfg!(windows) {
         println!("cargo:rustc-link-search=native={}/{}", dst.display(), build_type);
         println!("cargo:rustc-link-lib=dylib=mincore");
+    } else {
+        println!("cargo:rustc-link-search=native={}", dst.display());
+        println!("cargo:rustc-link-lib=dylib=stdc++");
     }
 
     if cfg!(target_os = "linux") {
