@@ -68,10 +68,6 @@ namespace snmalloc
      * Decommit superslabs when they are entirely empty.
      */
     DecommitSuper,
-    /**
-     * Decommit all slabs once they are empty.
-     */
-    DecommitAll,
     /**
      * Decommit superslabs only when we are informed of memory pressure by the
      * OS, do not decommit anything in normal operation.
