@@ -107,9 +107,7 @@ extern "C"
   SNMALLOC_EXPORT void*
     SNMALLOC_NAME_MANGLE(memalign)(size_t alignment, size_t size)
   {
-    if (
-      (alignment == 0) || (alignment == size_t(-1)) ||
-      (alignment > SUPERSLAB_SIZE))
+    if ((alignment == 0) || (alignment == size_t(-1)))
     {
       errno = EINVAL;
       return nullptr;
